/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WebDavBrowser.
 *
 * The Initial Developer of the Original Code is
 *   Rafa Malinowski, Pozna Supercomputing And Networing Center
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// for file sending
const nsILocalFile = Components.interfaces.nsILocalFile;
const nsIFilePicker = Components.interfaces.nsIFilePicker;

var webDavBrowser;

function WebDavBrowser() {
}

/**
 * Class for displaying tree-view of WebDav resources.
 */
WebDavBrowser.prototype = {
	_webDavTreeView: null,
	_popupItem: null, // item clicked with RMB

	init: function() {
		this._webDavTreeView  = new WebDavTreeView(document);
		document.getElementById("webdav-browser-tree").view = this._webDavTreeView;
		
		var rootResource = Components.classes["@mozilla.org/network/webdav-resource;1"]
			.createInstance(Components.interfaces.nsIWebDavResource);

		// get resource requested in URL bar
		rootResource.observer = this;
		rootResource.init(null, window.location.host, window.location.pathname, true);
	},

	// rootResource.observer
	onDebugMessage: function(message) {
		alert(message);
	},
	
	onStatusChanged: function(resource) {
	},
	
	onLoaded: function(resource) {
		// we got resource requested in URL bar, make TreeItem and add it to Tree
		var webDavTreeItem = new WebDavTreeItem(this._webDavTreeView, null, resource);
		this._webDavTreeView.addRootItem(webDavTreeItem);
	},

	onRemoved: function(resource) {
	},
	
	onSubResourceAdded: function(resource, subResource) {
	},
	
	onSubResourcesLoaded: function(resource) {
	},

	onCreated: function(resource) {
	},

	onError: function(resource, action, errorCode) {
		var error;
		var reason;

		switch (action) {
			case nsIWebDavResource.WEBDAV_ACTION_LOAD:
				error = "Unable to load resource " + resource.path;
				break;
			case nsIWebDavResource.WEBDAV_ACTION_CREATE:
				error = "Unable to create resource in " + resource.path;
				break;
			case nsIWebDavResource.WEBDAV_ACTION_DELETE:
				error = "Unable to delete resource " + resource.path;
				break;
			case nsIWebDavResource.WEBDAV_ACTION_READ:
			default:
				return;
		}
		
		switch (errorCode) {
			case nsIWebDavResource.WEBDAV_ERROR_NO_ERROR:
				return;
				
			case nsIWebDavResource.WEBDAV_ERROR_UNKNOW:
				reason = "\nReason: unknow";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_SERVER_UNAVAILABLE:
				reason = "\nReason: server unavailable";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_INVALID_RESPONSE:
				reason = "\nReason: invalid response from server";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_BAD_REQUEST: // also hen MKCOL on non-collection resource<
				reason = "\nReason: bad request to server";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_INSUFFICENT_STORAGE:
				reason = "\nReason: insufficent storage on server";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_RESOURCE_ALREADY_EXISTS:
				reason = "\nReason: resource already exists";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_PARENT_COLLECTION_NOT_FOUND:
				reason = "\nReason: one of parent collections is missing";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_NOT_FOUND:
				reason = "\nReason: resource not found";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_FORBIDDEN:
				reason = "\nReason: access forbidden";
				break;
				
			default:
				reason = "\nReason: unknow";
				break;
		}
		
		alert(error + reason);
	},
	
	// <tree> and <popup> handling

	onDblClick: function(event) {
		this._webDavTreeView.onTreeDblClick(event.clientX, event.clientY);
	},
	
	onPopupShowing: function(event) {
		var popup = document.getElementById("tree-popup-menu");
		var row = this._webDavTreeView.tree.getRowAt(event.clientX, event.clientY);

		if (row == -1)
			return false;
		
		this._popupItem = this._webDavTreeView._items[row];
		if (this._popupItem.isCollection) {
			document.getElementById("tree-popup-menu-new-collection").setAttribute("disabled", false);
			document.getElementById("tree-popup-menu-add-file").setAttribute("disabled", false);
		} else {
			document.getElementById("tree-popup-menu-new-collection").setAttribute("disabled", true);
			document.getElementById("tree-popup-menu-add-file").setAttribute("disabled", true);
		}
	
		return true;
	},
	
	onOpenClicked: function(event) {
		if (this._popupItem)
			this._popupItem.open();
	},
	
	onCreateNewCollectionClicked: function(event) {
		if (!this._popupItem) // ignore when clicked on free space
			return;

		var subCollection = prompt("New collection name:");

		if (subCollection) {
			if (!this._popupItem.webDavResource.isNameValid(subCollection)) {
				alert("Invalid collection name");
				return;
			}
			
			this._popupItem.open();
			var subResource = this._popupItem.webDavResource.createSubResource(subCollection, true);
			
			if (subResource != null)
				subResource.createCollection(false);
			else
				alert("Unable to create collection " + subCollection);
		}
	},

	onAddFileClicked: function(event) {
		if (!this._popupItem) // ignore when clicked on free space
			return;

		try {
			// open file window and get some files
 			var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
 			fp.init(window, "", nsIFilePicker.modeOpenMultiple);
 			fp.appendFilters(nsIFilePicker.filterHTML | nsIFilePicker.filterText |
 		    	             nsIFilePicker.filterAll | nsIFilePicker.filterImages | nsIFilePicker.filterXML);

			if (fp.show() == nsIFilePicker.returnOK) {
				var files = fp.files;
				while (files.hasMoreElements()) {
					var file = files.getNext().QueryInterface(nsILocalFile);
					if (!this._popupItem.webDavResource.isNameValid(file.leafName)) {
						alert("Server does not support file name: " + file.leafName);
						continue;
					}

					var subResource = this._popupItem.webDavResource.createSubResource(file.leafName, false);
					if (subResource != null)
						subResource.createResourceFromFile(file, false);
					else
						alert("Unable to add file " + file.path);
				}
			}
		}
	 	catch(ex) {
	 		alert(ex);
 		}
	},
	
	onDeleteClicked: function(event) {
		var selection = webDavBrowser._webDavTreeView.selection;
		var rangeCount = selection.getRangeCount();
		var min = {};
		var max = {};

		// a trick to avoid removing subResources and then resources
		// only minimal set of resources will be removed

		for (var i = rangeCount - 1; i >= 0; i--) {
			selection.getRangeAt(i, min, max);
			for (var j = max.value; j >= min.value; j--)
				webDavBrowser._webDavTreeView._items[j].removing = true;
		}

		for (var i = rangeCount - 1; i >= 0; i--) {
			selection.getRangeAt(i, min, max);
			for (var j = max.value; j >= min.value; j--) {
				if (webDavBrowser._webDavTreeView._items[j].removing)
					webDavBrowser._webDavTreeView._items[j].webDavResource.remove(false);
			}
		}
	}
};

function WebDavBrowserStartup() {
	webDavBrowser = new WebDavBrowser();
	webDavBrowser.init();
}
